@@ -7,7 +7,9 @@
 #include <grp.h>
 #include <shadow.h>
 #include <time.h>
+#include <dirent.h>
 
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 
@@ -15,7 +17,7 @@
 #include "webserver.h"
 #include "WebPages.h"
 
-#define VERSION 2.4
+#define VERSION 2.5
 
 extern char *getfqdn(char *);
 extern void trim(char *);
@@ -27,7 +29,7 @@ extern char *getconfstring(char *string, char *file,int length);
 extern int regmatch(const char *, const char *);
 extern int regmatchi(const char *, const char *);
 extern int getport(char *string);
-extern int splitobjective(char *, char *, char *, char *, int *, int *);
+extern int splitobjective(char *, char *, char *, char *, int *, int *, int *);
 extern void trimallwhitespace(char *);
 extern int Send(int,char *,int);
 
@@ -68,6 +70,10 @@ int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
 			DisplayTextHeader(http_message_socket);
 			ShowGroupMembers(http_message_socket);
 			return (-1);
+		//} else if (!strcmp(HTTPBuffer, "/Jass") && ArgPosition!=HTTPBuffer) {
+			// If we have an argument to the JASS Stuff..
+		//	returncode=RunJass(http_message_socket,HTTPBuffer);
+		//	return (-1);
 		}
 
 		returncode = DefaultHeader(HTTPBuffer, pBuffer, size);
@@ -104,10 +110,19 @@ int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
 				    http_listen_socket, http_message_socket);
 		} else if (!strncmp(HTTPBuffer, "/setnetwork", 11)) {
 			returncode += Network_Set(HTTPBuffer, pBuffer, psize);
-		} else if (!strncmp(HTTPBuffer, "/status", 6)) {
+		} else if (!strncmp(HTTPBuffer, "/status", 7)) {
 			returncode += Status_Page(ArgPosition, pBuffer, psize);
+		} else if (!strncmp(HTTPBuffer, "/Jass", 5)) {
+			returncode = RunJass(http_message_socket,HTTPBuffer,pBuffer,psize);
+			// If there were no errors, return quicky.
+			// Otherwise, display the message/error appropriately.
+			if(returncode==0) {
+				return(-1);
+			}
+
+			returncode=strlen(HTTPOutputBuffer);
 		} else {
-			returncode = +Status_Page(ArgPosition, pBuffer, psize);
+			returncode+=Status_Page(ArgPosition, pBuffer, psize);
 		}
 
 		pBuffer = HTTPOutputBuffer;
@@ -128,7 +143,7 @@ int Status_Page(char *source, char *dest, int size)
 {
 	if(source && dest && size) {
 		strncpy(dest,
-			"<HTML><BODY><H1><CENTER>Welcome to SNARE for Solaris version 2.4</H1><P>Please select from the menu on the left.</CENTER><p>",
+			"<HTML><BODY><H1><CENTER>Welcome to SNARE for Solaris version 2.5</H1><P>Please select from the menu on the left.</CENTER><p>",
 			size);
 
 		// Ok.. we're cheating here. The only way we can get here is if we're running.
@@ -342,12 +357,141 @@ int ShowGroupMembers(int http_socket)
 	return (1);
 }
 
+// Run the Solaris JASS tool, if available.
+int RunJass(int http_socket, char *source, char *dest, int size)
+{
+	char *psource = source;
+	char Variable[100], Argument[100];
+	char driver[256]="";
+	int retval=0;
+	char Text[8192]="";
+	DIR *dir;
+	struct dirent *entry;
+	struct stat status;
+	int drivercount=0;
+	char filename[2048];
+	char exec_line[1024];
+	FILE *jass_pipe;
+
+	if(!file_exists("/opt/SUNWjass/bin/jass-execute")) {
+		strncpy(dest,"<b>The <a href=\"http://www.google.com/search?q=jass+solaris\">Solaris JASS</a> application does not seem to be installed in the /opt/SUNWjass directory. Cannot continue.<br><br>Please see the Sun Web Site for download details.</b><br>",size);
+		return(-1);
+	}
+		
+	while ((psource =
+		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
+				sizeof (Argument))) != (char *) NULL) {
+		if (strstr(Variable, "driver") != NULL) {
+			strncpy(driver, Argument, sizeof (driver));
+		}
+	}
+
+	if(strlen(driver)) {
+		char *pdriver=driver;
+		int validfilename=1;
+		// Basic filename sanity checks
+		while(*pdriver) {
+			if(!((*pdriver >= 'a' && *pdriver <= 'z') ||
+			   (*pdriver >= 'A' && *pdriver <= 'Z') ||
+			   (*pdriver >= '0' && *pdriver <= '9') ||
+			   *pdriver == '-' || *pdriver == '_')) {
+				validfilename=0;
+			}
+			pdriver++;
+		}
+		if(!validfilename) {
+			strncpy(dest,"JASS: Invalid driver name. Drivers must only be composed of the following characters: a-zA-Z0-9-_<br>",size);
+			return(-1);
+			
+		}
+		snprintf(filename,sizeof(filename),"/opt/SUNWjass/Drivers/%s.driver",driver);
+		if(!file_exists(filename)) {
+			strncpy(dest,"JASS: Invalid driver name. Could not find that driver name in the /opt/SUNWjass/Drivers directory.<br>",size);
+			return(-1);
+		}
+	} else {
+		// HERE: Interrogate the driver dir, list files.
+		if(!dir_exists("/opt/SUNWjass/Drivers")) {
+			strncpy(dest,"JASS: The JASS Drivers directory cannot be found in /opt/SUNWjass/Drivers. Cannot continue.<br><br>Please see the Sun Web Site for download details.<br>",size);
+			return(-1);
+		}
+
+		if((dir = opendir("/opt/SUNWjass/Drivers")) == NULL) {
+			strncpy(dest,"JASS: The JASS Drivers directory cannot be opened. Cannot continue.<br><br>Please see the Sun Web Site for download details.<br>",size);
+			return(-1);
+		}
+		strncpy(Text,"<b>The <a href=\"http://www.google.com/search?q=jass+solaris\">Solaris JASS</a> tool is a security configuration and evaluation tool. Snare can call JASS in evaluation mode using the following drivers:</b><br><ul>",sizeof(Text));
+		if(size >= 1) {
+			strncpy(dest,Text,size);
+			size-=strlen(Text);
+			dest+=strlen(Text);
+		}
+		
+		while((entry=readdir(dir)) != NULL) {
+			snprintf(filename,sizeof(filename),"%s/%s","/opt/SUNWjass/Drivers",entry->d_name);
+			if(lstat(filename,&status) != -1) {
+				if(S_ISREG(status.st_mode)) {
+					if(!strcmp(&entry->d_name[strlen(entry->d_name)-7],".driver")) {
+						strncpy(driver,entry->d_name,sizeof(driver));
+						// Kill the .driver bit.
+						driver[strlen(driver)-7]='\0';
+						snprintf(Text,sizeof(Text),"<li><A HREF=\"/Jass?driver=%s\">%s</a><br>\n",driver,driver);
+						if(size >= 1) {
+							strncpy(dest,Text,size);
+							size-=strlen(Text);
+							dest+=strlen(Text);
+						}
+						drivercount++;
+					}
+				}
+			}
+		}
+		closedir(dir);
+		if(!drivercount) {
+			strncpy(dest,"JASS: The JASS Drivers directory does not seem to contain any driver files. Cannot continue.<br><br>Please see the Sun Web Site for download details.<br>",size);
+		} else {
+			strncpy(Text,"</ul>\n",sizeof(Text));
+			if(size >= 1) {
+				strncpy(dest,Text,size);
+				size-=strlen(Text);
+				dest+=strlen(Text);
+			}
+		}
+		return(-1);
+	}
+
+
+	// If we are here, we need to display the text header.
+
+	DisplayTextHeader(http_socket);
+
+	snprintf(exec_line,sizeof(exec_line),"cd /opt/SUNWjass;/opt/SUNWjass/bin/jass-execute -a %s.driver",driver);
+	jass_pipe=popen(exec_line,"r");
+	if(jass_pipe == NULL) {
+		strncpy(Text,"Cannot Execute the jass-execute binary using the supplied driver.<br>",sizeof(Text));
+		retval+=Send(http_socket,Text,strlen(Text));
+		return(0);
+	}
+	while(fgets(Text,sizeof(Text),jass_pipe) != NULL) {
+		retval+=Send(http_socket,Text,strlen(Text));
+	}
+	pclose(jass_pipe);
+
+	retval += Send(http_socket, "\n", strlen("\n"));
+
+	return(0);
+}
+
 int Network_Config(char *source, char *dest, int size)
 {
 	struct Reg_Config config_struct;
 	int dw_config_error;
 	char str_DestPort[10];
 	char str_conferr[10], str_neterr[10];
+	int SyslogPriority, SyslogFacility;
+	char *str_facility[] = {"Kernel","User","Mail","Daemon","Auth","Syslog","Lpr","News","UUCP","Cron","Authpriv","Ftp","Local0","Local1","Local2","Local3","Local4","Local5","Local6","Local7"};
+	char *str_priority[] = {"Emergency","Alert","Critical","Error","Warning","Notice","Information","Debug"};
+	int i;
 
 	if(!source || !dest || !size) {
 		return(0);
@@ -397,7 +541,7 @@ int Network_Config(char *source, char *dest, int size)
 	strncat(dest, "\"></td></tr>", size - strlen(dest));
 
 	strncat(dest,
-		"<tr bgcolor=#FFFFCC><td>Destination UDP Port</td><td><input type=text name=dw_DestPort size=8 value=\"",
+		"<tr bgcolor=#FFFFCC><td>Destination UDP Port (514 for syslog)</td><td><input type=text name=dw_DestPort size=8 value=\"",
 		size - strlen(dest));
 
 	if (config_struct.dw_DestPort) {
@@ -423,22 +567,58 @@ int Network_Config(char *source, char *dest, int size)
 		strncat(dest, " checked", size - strlen(dest));
 	strncat(dest, "></td></tr>", size - strlen(dest));
 
+
+	SyslogPriority = config_struct.dw_Syslog & 7;
+	SyslogFacility = config_struct.dw_Syslog >> 3;
+	if(SyslogFacility > 11) {
+		SyslogFacility -= 4;
+	}
+
+	strncat(dest,"<tr bgcolor=#FFFFCC><td>SYSLOG Facility (optional) </td><td><select name=SyslogFacility>",size - strlen(dest));
+        for (i = 0; i < 19; i++)
+        {
+                strncat(dest,"<option",size - strlen(dest));
+                if (i == SyslogFacility)
+                        strncat(dest," selected>",size - strlen(dest));
+                else
+                        strncat(dest,">",size - strlen(dest));
+                strncat(dest,str_facility[i],size - strlen(dest));
+        }
+	strncat(dest,"</select></td></tr>",size - strlen(dest));
+
+	strncat(dest,"<tr bgcolor=#FFFFBB><td>SYSLOG Priority (optional) </td><td><select name=SyslogPriority>",size - strlen(dest));
+        for (i = 0; i < 8; i++)
+        {
+                strncat(dest,"<option",size - strlen(dest));
+                if (i == SyslogPriority)
+                        strncat(dest," selected>",size - strlen(dest));
+                else
+                        strncat(dest,">",size - strlen(dest));
+                strncat(dest,str_priority[i],size - strlen(dest));
+        }
+        strncat(dest,"</select></td></tr>",size - strlen(dest));
+
 	strncat(dest, "</table><br>", size - strlen(dest));
 	strncat(dest, "<input type=submit value=\"Change Configuration\">    ",
 		size - strlen(dest));
 	strncat(dest, "<input type=reset value=\"Reset Form\"></form>",
 		size - strlen(dest));
 
+
 	return (0);
 }
 
 int Network_Set(char *source, char *dest, int size)
 {
 	char *psource = source;
-	char Variable[100], web_port[100];
+	char Variable[100], web_port[100],syslog_fac[100],syslog_pri[100];
 	char Argument[100];
 	struct Reg_Config config_struct;
-	int dw_error_config = 0;
+	int dw_error_config = 0,i=0;
+	char *str_facility[] = {"Kernel","User","Mail","Daemon","Auth","Syslog","Lpr","News","UUCP","Cron","Authpriv","Ftp","Local0","Local1","Local2","Local3","Local4","Local5","Local6","Local7"};
+        char *str_priority[] = {"Emergency","Alert","Critical","Error","Warning","Notice","Information","Debug"};
+	int dw_SyslogClass = 0;
+	
 
 	// This function will display that the remote audit configurations have been changed, or there have been errors
 	strncpy(dest, "<h1><center>SNARE Configuration</h1>", size);
@@ -490,8 +670,28 @@ int Network_Set(char *source, char *dest, int size)
 				config_struct.dw_stdout = 0;
 			}
 		}
+		if (strstr(Variable,"SyslogPriority") != NULL) {
+                        strncpy(syslog_pri,Argument,sizeof(syslog_pri));
+                }
+                if (strstr(Variable,"SyslogFacility") != NULL) {
+                        strncpy(syslog_fac,Argument,sizeof(syslog_fac));
+                }
 	}
 
+	for (i = 0; i < 8; i++) {
+                if (strstr(syslog_pri,str_priority[i]) != NULL)
+                        config_struct.dw_Syslog = i;
+        }
+        for (i = 0; i < 19; i++)
+        {
+                if (strstr(syslog_fac,str_facility[i]) != NULL)
+                        dw_SyslogClass = i;
+        }
+	if(dw_SyslogClass > 11) {
+                dw_SyslogClass = dw_SyslogClass + 4;
+        }
+        config_struct.dw_Syslog = config_struct.dw_Syslog | (dw_SyslogClass << 3);
+
 	config_struct.dw_DestPort = atoi(web_port);
 	if (strlen(config_struct.str_NetworkDestination)
 	    && ((config_struct.dw_DestPort < 1)
@@ -582,6 +782,11 @@ int Network_Set(char *source, char *dest, int size)
 							fprintf(configfile,
 								"	file=stdout\n");
 						}
+						if(config_struct.dw_Syslog) {
+							fprintf(configfile,
+								"	syslog=%d\n",config_struct.dw_Syslog);
+						}
+						fprintf(configfile,"\n");
 						skip = 1;
 					} else {
 						skip = 0;
@@ -893,29 +1098,40 @@ int Remote_Set(char *source, char *dest, int size)
 								remote_struct.
 								str_RestrictIP);
 						}
+						// Reject passwords with backslashes.
 						if (strlen
 						    (remote_struct.str_Password)
 						    && remote_struct.
 						    dw_Password) {
-							if(strcmp(remote_struct.str_Password,OldPassword)) {
-								// The password has changed. Encrypt it.
-								char *tpass;
-								tpass=crypt(remote_struct.str_Password,SALT);
-								if(tpass) {
-									fprintf(configfile,
-										"	accesskey=%s\n",
-										tpass);
+							if(strstr(remote_struct.str_Password,"\\")) {
+								strncat(dest,
+									"Sorry, passwords with the backslash character are not allowed.<br>Please use the Back Button, and try another password.<br> The other ",
+									size - strlen(dest));
+								fprintf(configfile,
+									"	accesskey=%s\n",
+									OldPassword);
+
+							} else {
+								if(strcmp(remote_struct.str_Password,OldPassword)) {
+									char *tpass;
+									// The password has changed. Encrypt it.
+									tpass=crypt(remote_struct.str_Password,SALT);
+									if(tpass) {
+										fprintf(configfile,
+											"	accesskey=%s\n",
+											tpass);
+									} else {
+										// Crypt failed. Fallback to whatever the user had before.
+										fprintf(configfile,
+											"	accesskey=%s\n",
+											remote_struct.str_Password);
+									}
 								} else {
-									// Crypt failed. Fallback to whatever the user had before.
+									// Otherwise, leave it in peace
 									fprintf(configfile,
 										"	accesskey=%s\n",
 										remote_struct.str_Password);
 								}
-							} else {
-								// Otherwise, leave it in peace
-								fprintf(configfile,
-									"	accesskey=%s\n",
-									remote_struct.str_Password);
 							}
 						}
 						skip = 1;
@@ -975,7 +1191,7 @@ int Remote_Set(char *source, char *dest, int size)
 
 		if (dw_error != 0) {
 			strncat(dest,
-				"Remote Control Values have NOT been changed. Report error: ",
+				"Remote Control Values have NOT been changed. Report an error.",
 				size - strlen(dest));
 		} else {
 			strncat(dest,
@@ -1182,8 +1398,11 @@ int Objective_Display(char *source, char *dest, int size)
 			strncpy(reg_objective.str_eventid_match, LOGON_LOGOFF_EVENTS,
 				sizeof (reg_objective.str_eventid_match));
 			strncpy(reg_objective.str_general_match,
-				"Enter the non-header search term here",
+				".*",
 				sizeof (reg_objective.str_general_match));
+			strncpy(reg_objective.str_general_match_type,
+				"Any",
+				sizeof (reg_objective.str_general_match_type));
 			strncpy(reg_objective.str_user_match_type,
 				"Any",
 				sizeof (reg_objective.str_user_match_type));
@@ -1309,6 +1528,36 @@ int Objective_Display(char *source, char *dest, int size)
 			size - strlen(dest));
 		strncat(dest, "\"></td></tr>", size - strlen(dest));
 
+
+		strncat(dest,
+			"<tr bgcolor=#FFFFCC><td>Select the General Match Type</td><td>",
+			size - strlen(dest));
+		strncat(dest,
+			"<input type=radio name=str_general_match_type value=Any",
+			size - strlen(dest));
+
+		if (strcmp(reg_objective.str_general_match, ".*") ==
+		    NULL) {
+			strncat(dest, " checked", size - strlen(dest));
+			strncpy(reg_objective.str_general_match_type,"Any",sizeof(reg_objective.str_general_match_type));
+		}
+		strncat(dest, ">Match Any String    ", size - strlen(dest));
+		strncat(dest,
+			"<input type=radio name=str_general_match_type value=Include",
+			size - strlen(dest));
+		if (strstr(reg_objective.str_general_match_type, "Include") !=
+		    NULL) {
+			strncat(dest, " checked", size - strlen(dest));
+		}
+		strncat(dest,
+			">Include    <input type=radio name=str_general_match_type value=Exclude",
+			size - strlen(dest));
+
+		if (strstr(reg_objective.str_general_match_type, "Exclude") !=
+		    NULL)
+			strncat(dest, " checked", size - strlen(dest));
+		strncat(dest, ">Exclude    </td></tr>", size - strlen(dest));
+
 		strncat(dest,
 			"<tr bgcolor=#FFFFBB><td>Search Term<br><i>(regular expression)</i></td><td><input type=text name=str_general_match size=50 value=\"",
 			size - strlen(dest));
@@ -1316,6 +1565,11 @@ int Objective_Display(char *source, char *dest, int size)
 			size - strlen(dest));
 		strncat(dest, "\"></td></tr>", size - strlen(dest));
 
+
+
+
+
+
 		strncat(dest,
 			"<tr bgcolor=#FFFFCC><td>Select the User Match Type</td><td>",
 			size - strlen(dest));
@@ -1633,7 +1887,6 @@ int Objective_Result(char *source, char *dest, int size)
 			strncpy(reg_objective.str_general_match, Argument,
 				sizeof (reg_objective.str_general_match));
 		}
-
 		if (strstr(Variable, "str_event_type_succ") != NULL) {
 			if (i_event_type_set == 1) {
 				// A token has already been added
@@ -1683,6 +1936,10 @@ int Objective_Result(char *source, char *dest, int size)
 			strncpy(reg_objective.str_user_match_type, Argument,
 				sizeof (reg_objective.str_user_match_type));
 		}
+		if (strstr(Variable, "str_general_match_type") != NULL) {
+			strncpy(reg_objective.str_general_match_type, Argument,
+				sizeof (reg_objective.str_general_match_type));
+		}
 	}
 
 	if (!i_event_type_set) {
@@ -1791,6 +2048,7 @@ int Objective_Result(char *source, char *dest, int size)
 						    CONFIG_OBJECTIVES) {
 							int critic = 0;
 							char userstring[8];
+							char generalstring[8];
 							// WRITE OUT NEW OBJECTIVE HERE
 							if (strstr
 							    (reg_objective.
@@ -1827,6 +2085,16 @@ int Objective_Result(char *source, char *dest, int size)
 								    CRITICALITY_CLEAR;
 							}
 
+							if (strstr(reg_objective.str_general_match_type,"Any") != NULL) {
+								strncpy(generalstring, "match", sizeof(generalstring));
+								strncpy(reg_objective.str_general_match,".*",sizeof(reg_objective.str_general_match));
+							} else {
+								if (strstr(reg_objective.str_general_match_type,"Include") != NULL) {
+									strncpy(generalstring, "match", sizeof(generalstring));
+								} else {
+									strncpy(generalstring, "match!", sizeof(generalstring));
+								}
+							}
 
 							if (strstr(reg_objective.str_user_match_type,"Any") != NULL) {
 								strncpy(userstring, "user", sizeof(userstring));
@@ -1853,12 +2121,13 @@ int Objective_Result(char *source, char *dest, int size)
 							}
 
 							fprintf(configfile,
-								"	criticality=%d	event=%s	return=%s	%s=%s	match=%s\n",
+								"	criticality=%d	event=%s	return=%s	%s=%s	%s=%s\n",
 								critic,
 								reg_objective.str_eventid_match,
 								reg_objective.str_event_type,
 								userstring,
 								reg_objective.str_user_match,
+								generalstring,
 								reg_objective.str_general_match);
 
 							wroteconfig = 1;
@@ -1884,6 +2153,7 @@ int Objective_Result(char *source, char *dest, int size)
 					// Must not have been an objective header in the file...
 					int critic = 0;
 					char userstring[8];
+					char generalstring[8];
 					// WRITE OUT NEW OBJECTIVE HERE
 					if (strstr
 					    (reg_objective.str_critic,
@@ -1908,6 +2178,18 @@ int Objective_Result(char *source, char *dest, int size)
 						critic = CRITICALITY_CLEAR;
 					}
 
+					if (strstr(reg_objective.str_general_match_type,"Any") != NULL) {
+						strncpy(generalstring, "match", sizeof(generalstring));
+						strncpy(reg_objective.str_general_match,".*",sizeof(reg_objective.str_general_match));
+					} else {
+						if (strstr(reg_objective.str_general_match_type,"Include") != NULL) {
+							strncpy(generalstring, "match", sizeof(generalstring));
+						} else {
+							strncpy(generalstring, "match!", sizeof(generalstring));
+						}
+					}
+
+
 					if (strstr(reg_objective.str_user_match_type,"Any") != NULL) {
 						strncpy(userstring, "user", sizeof(userstring));
 						strncpy(reg_objective.str_user_match,".*",sizeof(reg_objective.str_user_match));
@@ -1933,7 +2215,7 @@ int Objective_Result(char *source, char *dest, int size)
 					}
 
 					fprintf(configfile,
-						"\n[Objectives]\n	criticality=%d	event=%s	return=%s	%s=%s	match=%s\n",
+						"\n\n[Objectives]\n	criticality=%d	event=%s	return=%s	%s=%s	%s=%s\n",
 						critic,
 						reg_objective.
 						str_eventid_match,
@@ -1942,6 +2224,7 @@ int Objective_Result(char *source, char *dest, int size)
 						userstring,
 						reg_objective.
 						str_user_match,
+						generalstring,
 						reg_objective.
 						str_general_match);
 				}
@@ -1997,6 +2280,7 @@ int Objective_Result(char *source, char *dest, int size)
 							// Replace this objective with the new version.
 							int critic = 0;
 							char userstring[8];
+							char generalstring[8];
 							// WRITE OUT NEW OBJECTIVE HERE
 							if (strstr
 							    (reg_objective.
@@ -2033,6 +2317,19 @@ int Objective_Result(char *source, char *dest, int size)
 								    CRITICALITY_CLEAR;
 							}
 
+
+							if (strstr(reg_objective.str_general_match_type,"Any") != NULL) {
+								strncpy(generalstring, "match", sizeof(generalstring));
+								strncpy(reg_objective.str_general_match,".*",sizeof(reg_objective.str_general_match));
+							} else {
+								if (strstr(reg_objective.str_general_match_type,"Include") != NULL) {
+									strncpy(generalstring, "match", sizeof(generalstring));
+								} else {
+									strncpy(generalstring, "match!", sizeof(generalstring));
+								}
+							}
+
+
 							if (strstr(reg_objective.str_user_match_type,"Any") != NULL) {
 								strncpy(userstring, "user", sizeof(userstring));
 								strncpy(reg_objective.str_user_match,".*",sizeof(reg_objective.str_user_match));
@@ -2058,13 +2355,14 @@ int Objective_Result(char *source, char *dest, int size)
 							}
 
 							fprintf(configfile,
-								"	criticality=%d	event=%s	return=%s	%s=%s	match=%s\n",
+								"	criticality=%d	event=%s	return=%s	%s=%s	%s=%s\n",
 								critic,
 								reg_objective.
 								str_eventid_match,
 								reg_objective.str_event_type,
 								userstring,
 								reg_objective.str_user_match,
+								generalstring,
 								reg_objective.str_general_match);
 
 							position += size;
@@ -2196,7 +2494,8 @@ int DefaultHeader(char *source, char *dest, int size)
 		"<br><A HREF=\"/restart\" style=\"color:FFFFFF;text-decoration:none\">Apply the Latest Audit Configuration</A><br>"
 		"<br><br><font size=-2><A HREF=\"/Users\" style=\"color:FFFFFF;text-decoration:none\">Display a list of Users</A><br>"
 		"<br><A HREF=\"/Groups\" style=\"color:FFFFFF;text-decoration:none\">Display a list of Groups</A><br>"
-		"<br><A HREF=\"/GroupMembers\" style=\"color:FFFFFF;text-decoration:none\">Display a list of GroupMembers</A></font><br>"
+		"<br><A HREF=\"/GroupMembers\" style=\"color:FFFFFF;text-decoration:none\">Display a list of GroupMembers</A><br>"
+		"<br><br><font size=-2><A HREF=\"/Jass\" style=\"color:FFFFFF;text-decoration:none\">Execute the Sun JASS tool</A></font><br>"
 		"</b></font>"
 		"<br>"
 		"</div>"
@@ -2456,6 +2755,11 @@ int Read_Config_From_File(struct Reg_Config *config_struct)
 								"",
 								SIZE_OF_DESTINATION);
 						}
+					} else if (issyslog(inputbuffer)) {
+						config_struct->dw_Syslog=get_syslog_dest(inputbuffer);
+						if(!config_struct->dw_Syslog) {
+							config_struct->dw_Syslog=13;
+						}
 					} else if (isstdout(inputbuffer)) {
 						config_struct->dw_stdout = 1;
 					}
@@ -2645,6 +2949,7 @@ int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 	int criticality;
 	int returncode;
 	int excludeflag = 0;
+	int excludematchflag = 0;
 	//int returnflag;		// HERE: Still need to add in support for this.
 	//int notreturnflag = 0;	// HERE: Still need to add in support for this.
 
@@ -2661,10 +2966,22 @@ int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
 
 		if ((criticality =
 		     splitobjective(inputbuffer, event, user, path,
-				    &returncode, &excludeflag)) > -1) {
+				    &excludeflag, &excludematchflag, &returncode)) > -1) {
 			// add the objective to the linked list.
 			trimallwhitespace(event);
 
+			if (excludematchflag) {
+				strncpy(objective->str_general_match_type,
+					"Exclude",
+					sizeof (objective->
+						str_general_match_type));
+			} else {
+				strncpy(objective->str_general_match_type,
+					"Include",
+					sizeof (objective->
+						str_general_match_type));
+			}
+
 			strncpy(objective->str_general_match, path,
 				SIZE_OF_GENERALMATCH);
 			strncpy(objective->str_eventid_match, event,
@@ -2737,3 +3054,16 @@ int Close_Objective_File(FILE * configfile)
 {
 	return (fclose(configfile));
 }
+
+int dir_exists(const char *path) {
+	struct stat st;
+
+	return stat(path, &st) == 0 && S_ISDIR(st.st_mode);
+}
+
+int file_exists(const char *path)
+{
+	struct stat st;
+
+	return stat(path, &st) == 0;
+}
