@@ -1,6 +1,6 @@
 /**********************************************************
  *
- * Snare for Solaris version 2.4
+ * Snare for Solaris version 2.5
  *
  * Author: InterSect Alliance Pty Ltd
  *
@@ -122,6 +122,7 @@ void DebugMsg(const char* pszFormat, ...);
 // Linked List Functions
 static Node *head, *currentnode;
 static int AuditDestination=AUDIT_TO_STDOUT;            // Send to STDOUT unless otherwise stated.
+static int SyslogDestination=13;			// user.notice
 
 char hostid[MAX_HOSTID]="";
 int remote_allow=0;
@@ -848,13 +849,14 @@ int read_config_file(int shortread)
 					int eventnumber;
 					int returncode;
 					int excludeflag=0;
+					int excludematchflag=0;
 
 					// Do we need to skip over the objectives?
 					if(shortread==1) {
 						continue;
 					}
 					
-					if((criticality = splitobjective(inputbuffer,event,user,path,&returncode,&excludeflag)) > -1) {
+					if((criticality = splitobjective(inputbuffer,event,user,path,&excludeflag,&excludematchflag,&returncode)) > -1) {
 						// add the objective to the linked list.
 						trimallwhitespace(event);
 
@@ -894,7 +896,7 @@ int read_config_file(int shortread)
 							}
 
 							if((eventnumber = eventname2number(event2)) > -1) {
-								AddToList(event2,user,path,criticality,options,excludeflag,returncode);
+								AddToList(event2,user,path,criticality,options,excludeflag,excludematchflag,returncode);
 
 								// Also, ensure this event is turned on
 
@@ -902,7 +904,7 @@ int read_config_file(int shortread)
 
 							} else {
 								if(!strcmp(event,"*")) {
-									AddToList("AUDIT_ALL",user,path,criticality,"",excludeflag,returncode);
+									AddToList("AUDIT_ALL",user,path,criticality,"",excludeflag,excludematchflag,returncode);
 								}
 							}
 						}
@@ -918,6 +920,12 @@ int read_config_file(int shortread)
 						if(! open_audit_network(inputbuffer)) {
 							fprintf(stderr,"WARNING: Could not open the network specified in the audit configuration: %s - Auditing to STDOUT\n",inputbuffer);
 						}
+					} else if(issyslog(inputbuffer)) {
+						SyslogDestination=get_syslog_dest(inputbuffer);
+						if(!SyslogDestination) {
+							SyslogDestination=13;
+							fprintf(stderr,"WARNING: Could not establish the correct syslog destination from the audit configuration: %s - Sending to user.notice\n",inputbuffer);
+						}
 					}
 				} else if(headertype == CONFIG_HOSTID) {
 					gethostident(inputbuffer,hostid,MAX_HOSTID);
@@ -1001,6 +1009,14 @@ int isnetwork(char *string)
 	}	
 }
 
+int issyslog(char *string)
+{
+	if(regmatch(string,"^syslog=")) {
+		return(1);
+	}
+	return(0);
+}
+
 int open_audit_output(char *string)
 {
 	string+=strlen("file=");
@@ -1022,6 +1038,18 @@ int open_audit_output(char *string)
 	return(0);
 }
 
+// For the moment, require the web server to do the work of converting
+// destinations to numerics.
+int get_syslog_dest(char *string)
+{
+	int destination;
+	string+=strlen("syslog=");
+	destination=atoi(string);
+	if(destination > 0) {
+		return(destination);
+	}
+	return(0);
+}
 
 // Open the network - destination host and port
 int open_audit_network(char *string)
@@ -1085,6 +1113,9 @@ int open_audit_network(char *string)
 		AuditSocketName.sin_port = htons(port);
 		
 		AuditDestination |= AUDIT_TO_NETWORK;
+		if(port == 514) {
+			AuditDestination |= AUDIT_TO_SYSLOG;
+		}
 		return(1);
 	}
 
@@ -1291,7 +1322,7 @@ char * strip (char * string)
 
 // Take an identified objective line
 // NOTE: string, event, user and match must all be the same buffer size!
-int splitobjective(char *string, char *event, char *user, char *match, int *returncode, int *excludeflag)
+int splitobjective(char *string, char *event, char *user, char *match, int *excludeflag, int *excludematchflag, int *returncode)
 {
 	char *startevent, *startuser, *startmatch, *startcrit, *startreturn;
 	char *stringpointer, *endstring;
@@ -1321,7 +1352,14 @@ int splitobjective(char *string, char *event, char *user, char *match, int *retu
 		}
 	}
 	startmatch=strstr(string,"match=");
-	
+	if(startmatch == (char *)NULL) {
+		startmatch=strstr(string,"match!=");
+		if(startmatch!=(char *)NULL) {
+			// EXCLUDE rather than include.
+			*excludematchflag=1;
+		}
+	}
+
 	// string pointers for iteration.
 	stringpointer=string;
 	eventp=event; userp=user; matchp=match; critp=criticality;
@@ -1384,8 +1422,13 @@ int splitobjective(char *string, char *event, char *user, char *match, int *retu
 	*userp='\0';
 	// Remove extra whitespace at start and end.
 	trim(user);
-	
-	stringpointer=startmatch+strlen("match=");
+
+	if(*excludematchflag) {
+		stringpointer=startmatch+strlen("match!=");
+	} else {
+		stringpointer=startmatch+strlen("match=");
+	}
+
 	while(*stringpointer && (stringpointer!=startcrit && stringpointer != startreturn && stringpointer!=startevent && stringpointer!=startuser && stringpointer != endstring)) {
 		*matchp=*stringpointer;
 		stringpointer++;
@@ -1466,8 +1509,7 @@ int IsListEmpty(void)
         return 0;
 }
 
-// Node * AddToList(int eventnumber, char *username, char *path, int criticality, char *options, int excludeflag, int returncode)
-Node * AddToList(char *eventname, char *username, char *path, int criticality, char *options, int excludeflag, int returncode)
+Node * AddToList(char *eventname, char *username, char *path, int criticality, char *options, int excludeflag, int excludematchflag, int returncode)
 {
     Node *newNode=NULL;
     char comment[128];
@@ -1493,6 +1535,7 @@ Node * AddToList(char *eventname, char *username, char *path, int criticality, c
 	newNode->criticality=criticality;
 	newNode->returncode=returncode;
 	newNode->excludeflag=excludeflag;
+	newNode->excludematchflag=excludematchflag;
 
 	strncpy(newNode->username,username,MAX_USERREG);
         // REDRED: NOTE: Must free regexp in objective list at end!
@@ -1828,6 +1871,7 @@ int sendevent(char *string,int criticality)
 	static int snareseq=1;
 	char stringout[MAX_AUDITREC]="";
 	
+
 	// Audit record format, including host and SolarisBSM identifier string:
 	snprintf(stringout,MAX_AUDITREC,"%s\tSolarisBSM\t%d\t%s\tsnareseq,%d",hostid,criticality,string,snareseq);
 
@@ -1852,6 +1896,18 @@ int sendevent(char *string,int criticality)
 	}
 	
 	if(AuditDestination & AUDIT_TO_NETWORK) {
+		if(AuditDestination & AUDIT_TO_SYSLOG) {
+			// Do Syslog stuff here.
+			time_t currenttime;
+			struct tm *newtime;
+			char CurrentDate[16]="";
+
+			time(&currenttime);
+			newtime=localtime(&currenttime);
+			syslogdate(CurrentDate,newtime);
+
+			snprintf(stringout,MAX_AUDITREC,"<%d> %s %s SolarisBSM\t%d\t%s\tsnareseq,%d",SyslogDestination,CurrentDate,hostid,criticality,string,snareseq);
+		}
 		if(AuditSocket) {		
 			SendToSocket(stringout);
 		}
@@ -1875,23 +1931,8 @@ Node * CheckObjective(char *username,char *searchterm,char *eventname,char *opti
 
 		if((!strcmp(eventname,currentnode->event_name) || !strcmp(currentnode->event_name,"AUDIT_ALL")) && (returncode == currentnode->returncode || currentnode->returncode == RETURNCODE_ANY)) {
 			// Are we including users, or excluding.
-			if(currentnode->excludeflag) {
-				// REGMATCH REPLACEMENT CANDIDATE
-				// if(!(regmatch(username,currentnode->username)) && regmatch(searchterm,currentnode->path)) {
-				if(!(regmatchC(username,currentnode->usernameRE)) && regmatchC(searchterm,currentnode->pathRE)) {
-					// REGMATCH REPLACEMENT CANDIDATE
-					//if(regmatchi(options,currentnode->options)) {
-					if(regmatchC(options,currentnode->optionsRE)) {
-						NextItemInList();
-						return(currentnode);
-					}
-				}
-			} else {
-				// REGMATCH REPLACEMENT CANDIDATE
-				// if(regmatch(username,currentnode->username) && regmatch(searchterm,currentnode->path)) {
-				if(regmatchC(username,currentnode->usernameRE) && regmatchC(searchterm,currentnode->pathRE)) {
-					// REGMATCH REPLACEMENT CANDIDATE
-					//if(regmatchi(options,currentnode->options)) {
+			if(currentnode->excludeflag ^ regmatchC(username,currentnode->usernameRE)) {
+				if(currentnode->excludematchflag ^ regmatchC(searchterm,currentnode->pathRE)) {
 					if(regmatchC(options,currentnode->optionsRE)) {
 						NextItemInList();
 						return(currentnode);
@@ -2116,6 +2157,59 @@ char * FormatDelimiters(char *eventbuffer)
 	return(eventbuffer);
 }
 
+void syslogdate(char *sdate, struct tm *cdate)
+{
+	char Month[4];
+	char Date[3];
+	char Hour[3];
+	char Min[3];
+	char Sec[3];
+
+	if(!sdate || !cdate) return;
+
+	switch (cdate->tm_mon) {
+		case 0: strcpy(Month,"Jan"); break;
+		case 1: strcpy(Month,"Feb"); break;
+		case 2: strcpy(Month,"Mar"); break;
+		case 3: strcpy(Month,"Apr"); break;
+		case 4: strcpy(Month,"May"); break;
+		case 5: strcpy(Month,"Jun"); break;
+		case 6: strcpy(Month,"Jul"); break;
+		case 7: strcpy(Month,"Aug"); break;
+		case 8: strcpy(Month,"Sep"); break;
+		case 9: strcpy(Month,"Oct"); break;
+		case 10: strcpy(Month,"Nov"); break;
+		default: strcpy(Month,"Dec"); break;
+	}
+
+	if(cdate->tm_mday<10) {
+		snprintf(Date,3," %d\0",cdate->tm_mday);
+	} else {
+		snprintf(Date,3,"%d\0",cdate->tm_mday);
+	}
+
+	if(cdate->tm_hour<10) {
+		snprintf(Hour,3,"0%d\0",cdate->tm_hour);
+	} else {
+		snprintf(Hour,3,"%d\0",cdate->tm_hour);
+	}
+
+	if(cdate->tm_min<10) {
+		snprintf(Min,3,"0%d\0",cdate->tm_min);
+	} else {
+		snprintf(Min,3,"%d\0",cdate->tm_min);
+	}
+
+	if(cdate->tm_sec<10) {
+		snprintf(Sec,3,"0%d\0",cdate->tm_sec);
+	} else {
+		snprintf(Sec,3,"%d\0",cdate->tm_sec);
+	}
+
+	snprintf(sdate,16,"%s %s %s:%s:%s\0",Month,Date,Hour,Min,Sec);
+}
+
+
 void DebugMsg(const char* pszFormat, ...)
 {
 	char buf[8192]="";
